package mbs.views;

import javax.microedition.broadcast.esg.ServiceGuideData;
import javax.microedition.lcdui.*;
import javax.microedition.broadcast.platform.*;
import java.util.Vector;
import javax.microedition.broadcast.esg.ServiceGuideListener;
import mbs.controllers.ServiceGuideController;

/**
 * Platform view
 *
 * From this view it is possible to discover and select
 * platform providers
 */
public class PlatformView extends Form implements ViewBase,
        PlatformProviderSelectorListener, CommandListener, ItemCommandListener, ServiceGuideListener {
    // Commands

    private Command iCmdExit;
    private Command iCmdDiscover;
    private Command iCmdAbort;
    private Command iCmdSelect;
    //private Command iCmdBack;
    // Currently discovered platform providers
    private Vector iProviders;
    // Progress of the discovery
    private Gauge iProgress;
    private AlertNote iProgressNote;
    private Alert alert;
    // Choice group holding the found platform providers
    private ChoiceGroup iChoiceGroup;
    private boolean flag;
    int itemNum;
    private Object updateLocker;
    private ServiceGuideController iServiceGuideController;

    /**
     * Constructor
     */
    PlatformView() {
        super("Available Platforms");
        iCmdDiscover = new Command("Discover", Command.SCREEN, 2);
        iCmdSelect = new Command("Select", Command.SCREEN, 3);
        //iCmdBack = new Command("Back", Command.BACK, 2);
        iCmdExit = new Command("Exit", Command.EXIT, 1);
        iChoiceGroup = new ChoiceGroup("Providers", ChoiceGroup.EXCLUSIVE);
        iChoiceGroup.setDefaultCommand(iCmdSelect);
        iChoiceGroup.setItemCommandListener(this);
        flag = true;
        // Create progress note
        iCmdAbort = new Command("Cancel", Command.STOP, 3);
        // iProgress = new Gauge(null, false, 100, 0);
        iProgress = new Gauge(null, false, 100, Gauge.CONTINUOUS_RUNNING);

        iProviders = new Vector();
        iProgress.setLabel("Scanning...");
        itemNum = append(iProgress);
        iProgress.setValue(0);
        addCommand(iCmdExit);
        iServiceGuideController = ServiceGuideController.getInstance();
    }

    /**
     * Activates the view
     */
    public void activate() {
//        // Register this object to listen platform provider selector notifications
        PlatformProviderSelector.addListener(this);

    }

    /**
     * Deactivates the view
     */
    public void deactivate() {
        PlatformProviderSelector.removeListener(this);
    }

    /**
     * Returns the displayable of this view
     *
     * @return Displayable of this view
     */
    public Displayable getDisplayable() {
        return this;
    }

    /**
     * Sets the command listener for this view
     *
     * @param aListener The command listener listening commands for this view
     */
    public void setCommandListener(CommandListener aListener) {
        super.setCommandListener(aListener);
    }

    /**
     * Handles a command which is delegated to this object
     *
     * @param aCommand The Command to be handled
     * @param aDisplayable The displayable which initiated the command
     */
    public void handleCommand(Command aCommand, Displayable aDisplayable) {
        if (aCommand == iCmdDiscover) {
            // Remove any existing data from the providers lists
            deleteAll();

            iProgress.setLabel("Scanning...");
            itemNum = append(iProgress);
            iProgress.setValue(0);
            iProviders.removeAllElements();
            iChoiceGroup.deleteAll();
            addCommand(iCmdDiscover);
            addCommand(iCmdExit);
            // Start asynchronous discover process
            PlatformProviderSelector.discover();
         
        }
    }

    /**
     * From ItemCommandListener.
     */
    public void commandAction(Command aCommand, Item aItem) {
        if (aCommand == iCmdSelect) {
            ServiceGuideController.getInstance().deinitialize();
            handleSelectProvider();
        }
    }

    /**
     * Handles commands by internal view components of this view object.
     * These commands are provided by the associated progress note.
     * @param aCommand The Command to be handled
     * @param aDisplayable The displayable which initiated the command
     */
    public void commandAction(Command aCommand, Displayable aDisplay) {
        if (aCommand == iCmdAbort) {
            ViewHandler.getDisplay().setCurrent(this);
            PlatformProviderSelector.abort();
        }
    }

    /**
     * From PlatformProviderSelectorListener
     */
    public void update(int aProgress, int aFound, boolean aContinuing) {
        iProgress.setValue(aProgress);
        updateProviders();
        // Remove gauge
        if (aProgress >= 100) {
            ViewHandler.getDisplay().setCurrent(this);
            delete(itemNum);
            addCommand(iCmdDiscover);
            append(iChoiceGroup);
        }
    }

    /**
     * Handles select provider command
     */
    private void handleSelectProvider() {
        Thread selectThread = new Thread() {

            public void run() {
                int index = iChoiceGroup.getSelectedIndex();
                if (index != -1) {
                    PlatformProvider provider = (PlatformProvider) iProviders.elementAt(index);
                    // Start selection progress
                    AlertNote progressNote = new AlertNote(
                            "Selecting",
                            "Selecting platform: " + provider.toString(),
                            AlertType.INFO,
                            getDisplayable());

                    Gauge progress = new Gauge(null, false, Gauge.INDEFINITE, Gauge.CONTINUOUS_RUNNING);
                    progressNote.setIndicator(progress);
                    progressNote.addCommand(new Command("Hide", Command.STOP, 3));
                    progressNote.execute();
                    Exception exception = null;
                    try {
                        // Select provider
                        PlatformProviderSelector.select(provider);
                    } catch (Exception e) {
                        exception = e;
                    }
                    // Dismiss progress note
                    ViewHandler.getDisplay().setCurrent(getDisplayable());

                    if (exception != null) {
                        AlertNote alert = new AlertNote(
                                "Error",
                                "Cannot select provider: " + exception.getMessage(),
                                AlertType.ERROR,
                                getDisplayable());
                        alert.execute();
                    } else {
                        updateLocker = new Object();
                        // update the esg
                        iServiceGuideController.initialize();
                        iServiceGuideController.addListener((PlatformView) ViewHandler.getCurrentView());
                      //   iServiceGuideController.addListener(PlatformView.this);
                        iServiceGuideController.updateServiceGuide();
                        // wait for the completion of update esg
                        synchronized (updateLocker) {
                            try {
                                updateLocker.wait();
                                updateLocker = null;
                            } catch (InterruptedException ex) {
                                ex.printStackTrace();
                            }
                        }
                        ViewHandler.activateView(ViewHandler.PLATFORM_PROVIDER_VIEW_ID);
                    }
                }
            }
        };
        selectThread.start();
    }

    /**
     * Updates the list of available providers
     */
    private void updateProviders() {

        PlatformProvider[] providers =
                PlatformProviderSelector.getAvailableProviders();

        for (int i = 0; i < providers.length; i++) {
            if (!iProviders.contains(providers[i])) {
                iProviders.addElement(providers[i]);
                iChoiceGroup.append(providers[i].toString(), null);
            }
        }
    }

    public void serviceGuideUpdated(String aEvent, ServiceGuideData sgd, Object o) {
        if (aEvent.equalsIgnoreCase(
                ServiceGuideListener.SERVICE_GUIDE_UPDATE_COMPLETED)) {
            synchronized (updateLocker) {
                updateLocker.notify();
                iServiceGuideController.removeListener(this);
            }
        }
    }
}
