package mbs.views;

// EXTERNAL IMPORTS
import javax.microedition.lcdui.*;
import javax.microedition.broadcast.purchase.*;
import java.util.Vector;
import mbs.controllers.SubscriptionController;

/**
 * Shows information about service components.
 */
public class PriceOptionView extends List implements ViewBase, PurchaseListener
{
    SubscriptionController iController;
    // Available price options.
    Vector iPriceOptions;
    // Commands
    private Command iCmdRequestOptions;
    private Command iCmdPurchase;
    private Command iCmdBack;

    /**
     * Constructor
     */
    PriceOptionView()
    {
        super("Price options", List.IMPLICIT);

        iController = SubscriptionController.getInstance();
        iPriceOptions = new Vector(2);

        iCmdRequestOptions = new Command("Request price options", Command.SCREEN, 1);
        iCmdPurchase = new Command("Purchase", Command.SCREEN, 1);
        iCmdBack = new Command("Back", Command.BACK, 1);

        addCommand(iCmdRequestOptions);
        setSelectCommand(iCmdPurchase);
        addCommand(iCmdBack);
    }

    /**
     * Activates the view
     */
    public void activate()
    {
        reloadPriceOptions();
    }

    /**
     * Deactivates the view
     */
    public void deactivate()
    {
    }

    /**
     * Returns the displayable of this view
     *
     * @return Displayable of this view
     */
    public Displayable getDisplayable()
    {
        return this;
    }

    /**
     * Handles a command which is delegated to this object
     *
     * @param aCommand The Command to be handled
     * @param aDisplayable The displayable which initiated the command
     */
    public void handleCommand(Command aCommand, Displayable aDisplayable)
    {
        if (aCommand == iCmdBack)
        {
            ViewHandler.activatePreviousView();
            return;
        }


        PriceOption option = null;

        synchronized (iPriceOptions)
        {
            int index = getSelectedIndex();
            if (index >= 0 && index < iPriceOptions.size())
            {
                option = (PriceOption)iPriceOptions.elementAt(index);
            }
        }

        if (option == null)
        {
            // Nothing selected.
            return;
        }

        if (aCommand == iCmdPurchase)
        {
            purchase(option);
        }
        else if (aCommand == iCmdRequestOptions)
        {
            requestPriceOptions();
        }
    }

    /**
     * From PurchaseListener.
     */
    public void purchaseResponse(PurchaseObject aSource,
                                 String aEvent,
                                 Object aEventData)
    {
        System.out.println("PriceOptionView.purchaseResponse: " + aEvent);

        ViewBase currentView = ViewHandler.getCurrentView();
        Displayable current = currentView.getDisplayable();

        if (!aEvent.startsWith("Cancel"))
        {
            String message = aEvent;
            if (aEventData instanceof String)
            {
                message += ", " + aEventData;
                System.out.println("PriceOptionView.purchaseResponse, data=" +
                                   aEventData);
            }

            AlertNote note = new AlertNote("Purchase response",
                                           message,
                                           AlertType.INFO,
                                           current);
            note.execute();
        }

        if (aEvent.equalsIgnoreCase(PRICE_INFO_RECEIVED) ||
                aEvent.equalsIgnoreCase(PRICE_INFO_RECEIVAL_FAILED))
        {
            reloadPriceOptions();
        }
    }

    /**
     * Asks the price options from the controller and appends
     * them to the list.
     */
    private void reloadPriceOptions()
    {
        PriceOption[] priceOptions = iController.getPriceOptions();

        synchronized (iPriceOptions)
        {
            deleteAll();
            iPriceOptions.removeAllElements();

            for (int i = 0; i < priceOptions.length; i++)
            {
                iPriceOptions.addElement(priceOptions[ i ]);

                String temp = priceOptions[ i ].getPrice() + " (" +
                              priceOptions[ i ].getPriceDescription() + ")";

                append(temp, null);
            }
        }
    }

    /**
     * Starts to purchase the selected purchase object with
     * the given price option.
     */
    private void purchase(PriceOption aPriceOption)
    {
        PurchaseObject purchaseObject = iController.getSelectedPurchaseObject();
        purchaseObject.addPurchaseListener(this);
        iController.purchase(aPriceOption);
    }

    /**
     * Request price options for the selected purchase object.
     */
    private void requestPriceOptions()
    {
        PurchaseObject purchaseObject = iController.getSelectedPurchaseObject();
        purchaseObject.addPurchaseListener(this);
        iController.requestPriceOptions();
    }
}

// End of File
