package mbs.views;



import javax.microedition.lcdui.ChoiceGroup;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.Displayable;
import javax.microedition.broadcast.esg.ServiceGuideData;
import javax.microedition.broadcast.purchase.PurchaseObject;
import mbs.controllers.SubscriptionController;
/**
 * Displays information from the specified purchase object.
 */
public class PurchaseObjectDataInfo extends ServiceGuideDataInfo
{
    // Available channels. The first index means no channel.
    private ChoiceGroup iChannels;

    /**
     * Constructor
     */
    public PurchaseObjectDataInfo(
        String aTitle,
        Displayable aNextDisplayable,
        ServiceGuideData aData)
    {
        super(aTitle, aNextDisplayable, aData);
    }

    /**
     * Derived from ServiceGuideDataInfo.
     */
    public void commandAction(Command aCommand, Displayable aDisplayable)
    {
        if (aCommand == iCmdBack)
        {
            try
            {
                selectChannel();
            }
            catch (Throwable t)
            {
                t.printStackTrace();
            }
            super.commandAction(aCommand, aDisplayable);
        }
    }

    /**
     * Derived from ServiceGuideDataInfo.
     */
    protected void reloadMetadata()
    {
        if (iChannels == null)
        {
            iChannels =
                new ChoiceGroup("Purchase channel", ChoiceGroup.EXCLUSIVE);
        }

        deleteAll();

        loadChannelData();
        append(iChannels);

        refreshMetadata();
    }

    /**
     * Loads purchase channel data to the choice group.
     */
    private void loadChannelData()
    {
        iChannels.deleteAll();

        // The first index means no channel.
        iChannels.append("No channel used", null);

        // Add supported channels to the list.
        SubscriptionController controller = SubscriptionController.getInstance();
        String[] supportedChannels = controller.getSupportedChannels();

        for (int i = 0; i < supportedChannels.length; i++)
        {
            iChannels.append(supportedChannels[ i ], null);
        }

        // Select the channel that is returned by the object.
        PurchaseObject object = (PurchaseObject)iData;
        String channel = object.getPurchaseChannel();

        for (int i = 0; i < supportedChannels.length; i++)
        {
            if (supportedChannels[ i ].equals(channel))
            {
                iChannels.setSelectedIndex(i + 1, true);
            }
        }
    }

    /**
     * Stores the selected channel to the purchase object.
     * If no channel is selected, the choice is ignored.
     */
    private void selectChannel()
    {
        int selected = iChannels.getSelectedIndex();

        // Do not made any changes if the first index (no channel)
        // is selected. This is because the user cannot remove purchase
        // channel from the purchase object but one should be used.
        if (selected > 0)
        {
            String channel = iChannels.getString(selected);

            PurchaseObject object = (PurchaseObject)iData;
            object.setPurchaseChannel(channel);
        }

    }
}