package mbs.views;

// EXTERNAL IMPORTS
import java.io.IOException;
import javax.microedition.io.Connector;
import javax.microedition.io.InputConnection;
import javax.microedition.io.Connection;
import javax.microedition.lcdui.*;
import javax.microedition.broadcast.esg.*;
import javax.microedition.broadcast.connection.*;
import java.util.*;
import mbs.controllers.*;

public class ServicesView extends List
        implements ViewBase, ServiceGuideListener, BroadcastConnectionListener {
    // Service guide controller

    private ServiceGuideController iServiceGuideController;
    // Presentation controller
    private PresentationController iPresentationController;
    // Subscription controller
    private SubscriptionController iSubscriptionController;
    // List of available services
    private Vector iServices;
    // Hashtable of connections
    private Hashtable iConnections;
    // Commands
    private Command iCmdBack;
    private Command iCmdSelect;
    private Command iCmdExit;
    private Command iCmdUpdateESG;
    //private Command iCmdAbort;
    private Command iCmdManageSubscriptions;
//    private Gauge iProgress;
//    private AlertNote iProgressNote;
    // Initializer
    private Initializer iInitializer;
    private boolean flag;

    /**
     * Constructor
     */
    ServicesView() {
        super("Available Services", List.IMPLICIT);

        iCmdBack = new Command("Back", Command.BACK, 1);
        iCmdExit = new Command("Exit", Command.EXIT, 2);
        iCmdSelect = new Command("Select", Command.SCREEN, 1);
        iCmdUpdateESG = new Command("Force ESG update", Command.SCREEN, 2);
        iCmdManageSubscriptions = new Command("Subscriptions", Command.SCREEN, 2);

        addCommand(iCmdBack);
        addCommand(iCmdExit);
        addCommand(iCmdUpdateESG);
        addCommand(iCmdManageSubscriptions);
        setSelectCommand(iCmdSelect);

        iServiceGuideController = ServiceGuideController.getInstance();
        iPresentationController = PresentationController.getInstance();
        iServices = new Vector();
        iConnections = new Hashtable();

        // Initializer for full update
        iInitializer = new Initializer();
        iInitializer.start();

    }

    /**
     * From ServiceGuideListener
     */
    public void serviceGuideUpdated(
            String aEvent,
            ServiceGuideData aServiceGuideData,
            Object aEventData) {

        AlertNote note = null;
        ViewBase currentView = ViewHandler.getCurrentView();
        Displayable displayable = currentView.getDisplayable();

        if (aEvent.equalsIgnoreCase(ServiceGuideListener.NEW_SERVICE_LISTED)) {
            synchronized (this) {
                if (!iServices.contains(aServiceGuideData)) {
                    addService((Service) aServiceGuideData);
                }
            }
        } else if (aEvent.equalsIgnoreCase(ServiceGuideListener.SERVICE_DELETED)) {
            synchronized (this) {
                removeService((Service) aServiceGuideData);
            }
        } else if (aEvent.equalsIgnoreCase(ServiceGuideListener.SERVICE_GUIDE_BULK_CHANGED)) {
            requestUpdate();
        } else if (aEvent.equalsIgnoreCase(ServiceGuideListener.SERVICE_GUIDE_DELETED)) {
            synchronized (this) {
                iServices.removeAllElements();
                deleteAll();
                iConnections.clear();
            }
        }

        System.out.println("PlatformProviderView.serviceGuideUpdated() aEvent=" + aEvent);
        if (aEvent.equalsIgnoreCase(
                ServiceGuideListener.SERVICE_GUIDE_UPDATE_STARTED)) {
            note = new AlertNote("Service guide change",
                    "ESG update started",
                    AlertType.INFO,
                    displayable);
        } else if (aEvent.equalsIgnoreCase(
                ServiceGuideListener.SERVICE_GUIDE_UPDATE_COMPLETED)) {
            note = new AlertNote("Service guide change",
                    "ESG update completed",
                    AlertType.INFO,
                    displayable);
        } else if (aEvent.equalsIgnoreCase(
                ServiceGuideListener.SERVICE_GUIDE_UPDATE_FAILED)) {
            note = new AlertNote("Service guide change",
                    "ESG update failed!",
                    AlertType.ERROR,
                    displayable);
        } else if (aEvent.equalsIgnoreCase(
                ServiceGuideListener.SERVICE_GUIDE_DELETED)) {
            note = new AlertNote("Service guide change",
                    "ESG invalidated",
                    AlertType.INFO,
                    displayable);
            note.execute();
            return;
        }

        //  if (note != null) {
        note.execute();

    }

    /**
     * From BroadcastFileConnectionListener
     */
    public void update(
            BroadcastConnection aConnection,
            String aEvent,
            Object aEventData) {

        Service service = (Service) iConnections.get(aConnection);
        try {
            if (service != null && aEvent == BroadcastConnectionListener.AVAILABLE) {
                int serviceIndex = iServices.indexOf(service);
                set(serviceIndex, getString(serviceIndex),
                        readLogo(service, aConnection));
                aConnection.removeListener(this);
                // Remove from connections table
                iConnections.remove(aConnection);
            }
        } catch (Exception e) {
            e.printStackTrace();
            // Ignore
        }
    }

    /**
     * Activates the view
     */
    public void activate() {
        iSubscriptionController = SubscriptionController.getInstance();
        requestUpdate();
        iServiceGuideController.addListener(this);
    }

    /**
     * Deactivates the view
     */
    public void deactivate() {
        // Do not remove listener as the service guide might get invalidated
        iServiceGuideController.removeListener(this);
    }

    /**
     * Returns the displayable of this view
     *
     * @return Displayable of this view
     */
    public Displayable getDisplayable() {
        return this;
    }

    /**
     * Sets the command listener for this view
     *
     * @param aListener The command listener listening commands for this view
     */
    public void setCommandListener(CommandListener aListener) {
        super.setCommandListener(aListener);
    }

    /**
     * Handles a command which is delegated to this object
     *
     * @param aCommand The Command to be handled
     * @param aDisplayable The displayable which initiated the command
     */
    public void handleCommand(Command aCommand, Displayable aDisplayable) {
        if (aCommand == iCmdBack) {
            ViewHandler.activateView(ViewHandler.PLATFORM_VIEW_ID);
        }
        int index = getSelectedIndex();
        if (index < 0) {
            // Nothing selected.
            return;
        }
        Service service = (Service) iServices.elementAt(index);
        if (aCommand == iCmdSelect) {
            iPresentationController.prepare(service);
            ViewHandler.activateView(ViewHandler.VIDEO_CANVAS_VIEW_ID);
        } else if (aCommand == iCmdUpdateESG) {
            iServices.removeAllElements();
            deleteAll();

            iServiceGuideController.updateServiceGuide();//added
            ViewHandler.activateView(ViewHandler.SERVICES_VIEW_ID);
            setSelectedIndex(0, true);
        } else if (aCommand == iCmdManageSubscriptions) {
            ViewHandler.activateView(ViewHandler.SUBSCRIPTION_VIEW_ID);
        }
    }

    /**
     * Adds a new service
     *
     * @param aService The service to be added
     */
    private synchronized void addService(Service aService) {
        iServices.addElement(aService);
        String name = null;
        try {
            // List service names
            name = aService.getStringValue(CommonMetadataSet.SERVICE_NAME);

        } catch (DataUnavailableException due) {
            name = "(Unnamed)";

        } // Add default icon for now
        append(name, null);
        int index = iServices.size() - 1;
        setFont(
                index, ViewUtils.getDefaultFont());
    }

    private synchronized void removeService(Service aService) {
        int index = iServices.indexOf(aService);
        if (index != -1) {
            iServices.removeElement(aService);
            delete(index);
            // remove connection if available
            Enumeration elements = iConnections.keys();
            while (elements.hasMoreElements()) {
                BroadcastFileConnection con =
                        (BroadcastFileConnection) elements.nextElement();

                if (iConnections.get(con) == aService) {
                    iConnections.remove(con);
                    con.removeListener(this);
                    try {
                        con.close();
                    } catch (Exception e) {
                        // Ignore
                    }
                }
            }
        }
    }

    private synchronized void requestUpdate() {
        // Notify updater if not running

        try {
            synchronized (iInitializer) {
                iInitializer.notify();
            }
        } catch (Exception e) {
        }
    }

    private synchronized void updateLogos() {
        int count = iServices.size();
        for (int i = 0; i < count; i++) {
            Service service = (Service) iServices.elementAt(i);
            // Update only if no logo available

            if (getImage(i) == null && !iConnections.contains(service)) {
                try {
                    String logo = service.getStringValue(CommonMetadataSet.SERVICE_AUX_LOGO);
                    Connection logoConnection = Connector.open(logo, Connector.READ);
                    Image image = readLogo(service, logoConnection);
                    // Set image if it was available

                    if (image != null) {
                        String string = getString(i);
                        set(i, string, image);
                    }
                } catch (Exception e) {
                    // No image
                    e.printStackTrace();
                }
            }
        }
    }

    private Image readLogo(Service aService, Connection aConnection) {
        Image image = null;

        try {
            // Check if this is a broadcast file connection
            if (aConnection instanceof BroadcastFileConnection) {
                BroadcastFileConnection connection = (BroadcastFileConnection) aConnection;
                System.out.println("ServicesView.readLogo(), logo URL=" + connection.getURL());


                if (!connection.isAvailable()) {
                    iConnections.put(connection, aService);
                    connection.addListener(this);

                } else {
                    image = Image.createImage(connection.openInputStream());
                    connection.close();
                }
            } // Standard input connection
            else if (aConnection instanceof InputConnection) {
                InputConnection connection = (InputConnection) aConnection;
                image = Image.createImage(connection.openInputStream());
                connection.close();
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
        return image;
    }

    /**
     * Updates the services list
     */
    private synchronized void updateServices() {

        // Executes either query or returns all available services
        Vector services = iServiceGuideController.executeQuery();
        // Check which services have been removed
        Enumeration currentServices = iServices.elements();
        while (currentServices.hasMoreElements()) {
            Service service = (Service) currentServices.nextElement();
            if (!services.contains(service)) {
                removeService(service);
            }
        }
        // Check that which services are new
        Enumeration newServices = services.elements();
        while (newServices.hasMoreElements()) {
            Service service = (Service) newServices.nextElement();
            if (!iServices.contains(service)) {
                addService(service);
            }
        }
    }

    /**
     * Inner class which is used for service list initialization
     */
    private class Initializer extends Thread {

        /**
         * Constructor
         */
        public Initializer() {
        }

        /**
         * Sets the update mode
         */
        public void setUpdateMode() {
        }

        /**
         * Run method for this thread
         */
        public void run() {

            while (true) {

                // Processing done. Start wainting
                try {
                    synchronized (this) {
                        this.wait();
                    }
                    // Full update
                    updateServices();
                    updateLogos();
                } catch (Exception e) {
                    // Interrupted or service guide invalidated
                    e.printStackTrace();
                }
            }
        }
    }
}
// End of File

