package mbs.views;

import javax.microedition.broadcast.esg.DataUnavailableException;
import javax.microedition.lcdui.*;
import javax.microedition.media.MediaException;
import javax.microedition.media.Player;
import javax.microedition.media.control.VideoControl;
import javax.microedition.media.control.VolumeControl;

import javax.microedition.broadcast.esg.Service;
import javax.microedition.broadcast.esg.CommonMetadataSet;
import com.nokia.mid.media.AudioOutputControl;
import java.io.InputStream;
import java.util.Vector;
import mbs.controllers.*;
import com.nokia.mid.ui.TactileFeedback;

public class VideoCanvasView extends Canvas implements ViewBase, PresentationListener {

    private static final String PRIMARY = "Headset";
    private static final String SECONDARY = "Loudspeaker";
    private static final String NORMALSCREEN = "NormalScreen";
    private static final String FULLSCREEN = "FullScreen";
    private Command iCmdBack;
    private Command iCmdExit;
    private boolean iCanvasFullScreen = false;
    private ServiceGuideController iServiceGuide;
    private PresentationController iController;
    private Thread iThread;
    private Vector iServices;
    private Service iCurrentService;
    private boolean iFullscreen = false;
    private boolean iShowServiceComponents = false;
    private static final int THREAD_SLEEP_TIME = 2500;
    private static final int CTRL_WIDTH = 70;
    private static final int CTRL_HEIGHT = 70;
    private static final int MAX_GRID_CTRLS = 4;
    private static final int CHANNELPLUS_CTRL_LOC = 1;
    private static final int CHANNELMINUS_CTRL_LOC = 2;
    private static final int VOLUMEINCREASE_CTRL_LOC = 3;
    private static final int VOLUMEDECREASE_CTRL_LOC = 4;
    private static final String CHANNELPLUS_IMAGE = "/images/ChannelPlus.JPG";
    private static final String CHANNELMINUS_IMAGE = "/images/ChannelMinus.JPG";
    private static final String VOLUMEINCREASE_IMAGE = "/images/VolumeIncrease.JPG";
    private static final String VOLUMEDECREASE_IMAGE = "/images/VolumeDecrease.JPG";
    private static final String MAXIMIZE_IMAGE = "/images/Maximize.JPG";
    private static final String MINIMIZE_IMAGE = "/images/Minimize.JPG";
    private static final String LOUDSPEAKER_IMAGE = "/images/Loudspeaker.JPG";
    private static final String HEADSET_IMAGE = "/images/Headset.JPG";
    private static final String INFORMATION_IMAGE = "/images/Information.JPG";
    private static final int UNKNOWN_CMD = -1;
    private static final int CHANNEL_PLUS_CMD = 0;
    private static final int CHANNEL_MINUS_CMD = 1;
    private static final int VOL_INC_CMD = 2;
    private static final int VOL_DEC_CMD = 3;
    private static final int SPEAKER_CMD = 4;
    private static final int SCREEN_CMD = 5;
    private static final int INFO_CMD = 6;
    int canWidth;
    int canHeight;
    int grid;
    Image channelPlusImage;
    Image channelMinusImage;
    Image volumeIncreaseImage;
    Image volumeDecreaseImage;
    Image maximizeImage;
    Image loudspeakerImage;
    Player feedback;
    Image speakerImage;
    Image screenImage;
    Image informationImage;
    VideoControl videoCtrl;
    String speakerType;
    String screenType;
    TactileFeedback tactileFeedback;

    VideoCanvasView() {
        super();
        iCmdBack = new Command("Back", Command.BACK, 1);
        iCmdExit = new Command("Exit", Command.EXIT, 10);
        addCommand(iCmdBack);
        addCommand(iCmdExit);
        speakerType = PRIMARY;
        screenType = NORMALSCREEN;
        createImages();
        //repaint();
        initFeedback();

        iController = PresentationController.getInstance();
        iServiceGuide = ServiceGuideController.getInstance();
    }
////////////////////////////////

    protected void paint(Graphics g) {
        canWidth = getWidth();
        canHeight = getHeight();
        g.setColor(0, 0, 0);
        g.fillRect(0, 0, canWidth, canHeight);
        displayVideoControl();
        grid = (canHeight - MAX_GRID_CTRLS * CTRL_HEIGHT) / MAX_GRID_CTRLS;
        g.drawImage(channelPlusImage, canWidth - CTRL_WIDTH,
                (grid * CHANNELPLUS_CTRL_LOC) + (CTRL_HEIGHT * (CHANNELPLUS_CTRL_LOC - 1)),
                Graphics.LEFT | Graphics.TOP);
        g.drawImage(channelMinusImage, canWidth - CTRL_WIDTH,
                (grid * CHANNELMINUS_CTRL_LOC) + (CTRL_HEIGHT * (CHANNELMINUS_CTRL_LOC - 1)),
                Graphics.LEFT | Graphics.TOP);
        g.drawImage(volumeIncreaseImage, canWidth - CTRL_WIDTH,
                (grid * VOLUMEINCREASE_CTRL_LOC) + (CTRL_HEIGHT * (VOLUMEINCREASE_CTRL_LOC - 1)),
                Graphics.LEFT | Graphics.TOP);

        g.drawImage(volumeDecreaseImage, canWidth - CTRL_WIDTH,
                (grid * VOLUMEDECREASE_CTRL_LOC) + (CTRL_HEIGHT * (VOLUMEDECREASE_CTRL_LOC - 1)),
                Graphics.LEFT | Graphics.TOP);
        g.drawImage(screenImage, 0, (grid * 2),
                Graphics.LEFT | Graphics.TOP);
        g.drawImage(speakerImage, 0, (canHeight - CTRL_HEIGHT - (grid * 2)),
                Graphics.LEFT | Graphics.TOP);
        g.drawImage(informationImage, 0, (canHeight / 3),
                Graphics.LEFT | Graphics.TOP);
    }

    private void displayVideoControl() {

        try {
            videoCtrl.setDisplaySize(canWidth - CTRL_WIDTH * 2, canHeight);
            videoCtrl.setDisplayLocation(CTRL_WIDTH, 0);
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    private void createImages() {
        channelPlusImage = getImage(CHANNELPLUS_IMAGE);
        channelMinusImage = getImage(CHANNELMINUS_IMAGE);
        volumeIncreaseImage = getImage(VOLUMEINCREASE_IMAGE);
        volumeDecreaseImage = getImage(VOLUMEDECREASE_IMAGE);
        maximizeImage = getImage(MAXIMIZE_IMAGE);
        speakerImage = getImage(LOUDSPEAKER_IMAGE);
        screenImage = getImage(MAXIMIZE_IMAGE);
        informationImage = getImage(INFORMATION_IMAGE);
    }

    private Image getImage(String imageType) {
        Image image;
        try {
            image = Image.createImage(imageType);

        } catch (Exception ex) {
            ex.printStackTrace();
            image = null;
        }
        return image;
    }
    InputStream feedIS;
    private static final String FEEDBACK_SOUND = "/mbs/main/Feedback.wav";

    void initFeedback() {
        try {
            tactileFeedback = new TactileFeedback();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    void giveTactileFeedback() {
        try {
            tactileFeedback.directFeedback(TactileFeedback.FEEDBACK_STYLE_SENSITIVE);
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    protected void pointerPressed(int x, int y) {
        int cmd = getCommand(x, y);
        if (cmd != UNKNOWN_CMD) {
            try {
                giveTactileFeedback();
                //feedback.start();

            } catch (Exception ex) {
                ex.printStackTrace();
            }
        }
        commandAction(cmd);
    }

    private int getCommand(int x, int y) {
        Rectangle ctrlRect;
        System.out.println("Point : x - " + x + " ," + "y - " + y);

        ctrlRect = new Rectangle(canWidth - CTRL_WIDTH,
                (grid * CHANNELPLUS_CTRL_LOC) + (CTRL_HEIGHT * (CHANNELPLUS_CTRL_LOC - 1)),
                CTRL_WIDTH, CTRL_HEIGHT);
        if (ctrlRect.isContains(x, y)) {
            return CHANNEL_PLUS_CMD;
        }


        ctrlRect = new Rectangle(canWidth - CTRL_WIDTH,
                (grid * CHANNELMINUS_CTRL_LOC) + (CTRL_HEIGHT * (CHANNELMINUS_CTRL_LOC - 1)),
                CTRL_WIDTH, CTRL_HEIGHT);
        if (ctrlRect.isContains(x, y)) {
            return CHANNEL_MINUS_CMD;
        }
        ctrlRect = new Rectangle(canWidth - CTRL_WIDTH,
                (grid * VOLUMEINCREASE_CTRL_LOC) + (CTRL_HEIGHT * (VOLUMEINCREASE_CTRL_LOC - 1)),
                CTRL_WIDTH, CTRL_HEIGHT);
        if (ctrlRect.isContains(x, y)) {
            return VOL_INC_CMD;
        }
        ctrlRect = new Rectangle(canWidth - CTRL_WIDTH,
                (grid * VOLUMEDECREASE_CTRL_LOC) + (CTRL_HEIGHT * (VOLUMEDECREASE_CTRL_LOC - 1)),
                CTRL_WIDTH, CTRL_HEIGHT);
        if (ctrlRect.isContains(x, y)) {
            return VOL_DEC_CMD;
        }
        ctrlRect = new Rectangle(0, (grid * 2), CTRL_WIDTH, CTRL_HEIGHT);
        if (ctrlRect.isContains(x, y)) {
            return SCREEN_CMD;
        }

        ctrlRect = new Rectangle(0, (canHeight - CTRL_HEIGHT - (grid * 2)),
                CTRL_WIDTH, CTRL_HEIGHT);
        if (ctrlRect.isContains(x, y)) {
            return SPEAKER_CMD;
        }

        ctrlRect = new Rectangle(0, (canHeight / 3),
                CTRL_WIDTH, CTRL_HEIGHT);
        if (ctrlRect.isContains(x, y)) {
            return INFO_CMD;
        }

        return UNKNOWN_CMD;
    }

    private void commandAction(int command) {

        System.out.println("Command : " + command);

        switch (command) {
            case CHANNEL_PLUS_CMD: {
                int index = iServices.indexOf(iCurrentService);
                switchChannel(index >= 0 && index < iServices.size() - 1 ? index + 1 : 0);
                break;
            }
            case CHANNEL_MINUS_CMD: {
                int index = iServices.indexOf(iCurrentService);
                switchChannel(index > 0 ? index - 1 : iServices.size() - 1);
                break;
            }
            case VOL_INC_CMD: {
                VolumeControl volc = getVolumeControl();
                volc.setLevel(volc.getLevel() + 10);
                if (volc.getLevel() == 100) {

                    AlertNote iAlert = new AlertNote("Volume ", "Maximum Volume", AlertType.CONFIRMATION, this);
                    ViewHandler.getDisplay().setCurrent(iAlert);
                    break;
                } else {
                    updateTitle(volc.getLevel() + " ");
                    try {
                        Thread.sleep(500);
                    } catch (InterruptedException ex) {
                        ex.printStackTrace();
                    }
                    updateTitle(" ");
                    break;
                }
            }
            case VOL_DEC_CMD: {
                VolumeControl volc = getVolumeControl();
                volc.setLevel(volc.getLevel() - 10);
                if (volc.getLevel() == 0) {

                    AlertNote iAlert = new AlertNote("Volume ", "Volume Muted", AlertType.CONFIRMATION, this);
                    ViewHandler.getDisplay().setCurrent(iAlert);
                    break;
                } else {
                    updateTitle(volc.getLevel() + " ");
                    try {
                        Thread.sleep(500);
                    } catch (InterruptedException ex) {
                        ex.printStackTrace();
                    }
                    updateTitle(" ");
                    break;
                }
            }
            case SCREEN_CMD: {
                if (screenType.equals(FULLSCREEN)) {
                    screenImage = getImage(MAXIMIZE_IMAGE);
                    toggleCanvasFullscreen();
                    repaint();
                    screenType = NORMALSCREEN;
                    break;
                } else {
                    screenImage = getImage(MINIMIZE_IMAGE);
                    toggleCanvasFullscreen();
                    repaint();
                    screenType = FULLSCREEN;
                    break;
                }

            }
            case SPEAKER_CMD: {
                if (speakerType.equals(SECONDARY)) {
                    speakerImage = getImage(LOUDSPEAKER_IMAGE);
                    repaint();
                    Player[] players = iController.getCurrentPlayers();
                    for (int i = 0; i < players.length; i++) {
                        toggleAudioOutput(players[i]);
                    }

                    speakerType = PRIMARY;
                    break;
                } else {
                    speakerImage = getImage(HEADSET_IMAGE);
                    repaint();
                    Player[] players = iController.getCurrentPlayers();
                    for (int i = 0; i < players.length; i++) {
                        toggleAudioOutput(players[i]);
                    }
                    speakerType = SECONDARY;
                    break;
                }
            }
            case INFO_CMD: {
                int indexCurrent = iServices.indexOf(iCurrentService);
                Service service = (Service) iServices.elementAt(indexCurrent);
                String str;
                try {
                    str = service.getStringValue(CommonMetadataSet.SERVICE_NAME);
                    new ServiceGuideDataInfo(str, this, service);
                } catch (IllegalArgumentException ex) {
                    ex.printStackTrace();
                } catch (DataUnavailableException ex) {
                    ex.printStackTrace();
                }
            }
        }
    }

    class Rectangle {

        public int x;
        public int y;
        public int width;
        public int height;

        public Rectangle(int x, int y, int width, int height) {
            this.x = x;
            this.y = y;
            this.width = width;
            this.height = height;
        }

        public boolean isContains(Point p) {
            return isContains(p.x, p.y);

        }

        public boolean isContains(int x, int y) {
            if (((x >= this.x) && (x < (this.x + width)))
                    && ((y >= this.y) && (y < (this.y + height)))) {
                return true;
            } else {
                return false;
            }
        }

        class Point {

            public int x;
            public int y;
        }
    }

////////////////////////////////////////////////////
    public void activate() {
        iController.addListener(this);
        iController.start();
        iServices = iServiceGuide.executeQuery();
        iCurrentService = iController.getCurrentService();
        iThread = new Thread() {

            public void run() {
                while (iThread != null) {
                    try {
                        synchronized (this) {
                            updateTitle(" ");
                            sleep(
                                    THREAD_SLEEP_TIME);
                        }
                    } catch (Exception e) {
                        // Ignore
                    }
                }
            }
        };
        iThread.start();
    }

    private synchronized void updateTitle(String str) {
        String title = "";
        if (str.equals(" ")) {
            Service service = iController.getCurrentService();

            try {
                title = service.getStringValue(CommonMetadataSet.SERVICE_NAME);

            } catch (Exception e) {
                title = "No title ";
            }
            try {
                title += " (Signal:" + Integer.toString(iController.getSignalQuality()) + "%)";

            } catch (Exception e) {
                // Ignored intentionally
            }

        } else {
            title += " Vol: " + str;
        }
        setTitle(title);
    }

    public void deactivate() {
        if (!iShowServiceComponents) {
            iController.removeListener(this);
            iThread = null;
            iController.stop();
        }
        iShowServiceComponents = false;
    }

    public Displayable getDisplayable() {
        return this;
    }

    public void handleCommand(Command aCommand, Displayable aDisplayable) {
        if (aCommand == iCmdBack) {
            ViewHandler.activatePreviousView();
        }
    }

    public void notifyPresentationEvent(int aEventType, String aMessage) {
        if (aEventType == PresentationListener.EVENT_PRESENTING) {
            Player[] players = iController.getCurrentPlayers();
            // The players should all be realized at this point.
            // Initialize and display them.

            for (int i = 0; i < players.length; i++) {
                Player player = players[i];
                VideoControl control =
                        (VideoControl) player.getControl("VideoControl");

                AudioOutputControl aocontrol = (AudioOutputControl) player.getControl("com.nokia.mid.media.AudioOutputControl");
                aocontrol.setOutputMode(AudioOutputControl.PRIVATE);


                if (control != null) {
                    try {
                        control.initDisplayMode(VideoControl.USE_DIRECT_VIDEO, this);
                        // Show the video in the Canvas
                        control.setVisible(true);
                        control.setDisplayFullScreen(true);
                    } catch (MediaException ex) {
                        ex.printStackTrace();
                    }
                }
                VolumeControl vcontrol =
                        (VolumeControl) player.getControl("VolumeControl");
                if (vcontrol != null) {
                    vcontrol.setLevel(80);
                }
            }
        } else if (aEventType == PresentationListener.EVENT_ERROR) {
            AlertNote alert = new AlertNote("Error",
                    aMessage,
                    AlertType.ERROR,
                    this);
            alert.execute();
        }
    }

    private VideoControl getVideoControl() {
        VideoControl control = null;
        try {
            Player[] players = iController.getCurrentPlayers();
            for (int i = 0; i
                    < players.length; i++) {
                Player player = players[i];
                control = (VideoControl) player.getControl("VideoControl");
                if (control != null) {
                    break;
                }
            }
        } catch (Exception e) {
            // Incorrect state or no control
        }
        return control;
    }

    private VolumeControl getVolumeControl() {
        VolumeControl control = null;
        try {
            Player[] players = iController.getCurrentPlayers();
            for (int i = 0; i
                    < players.length; i++) {
                Player player = players[i];
                control = (VolumeControl) player.getControl("VolumeControl");
                if (control != null) {
                    break;
                }
            }
        } catch (Exception e) {
            // Incorrect state or no control
        }
        return control;
    }

    private void toggleCanvasFullscreen() {

        iCanvasFullScreen = !iCanvasFullScreen;
        setFullScreenMode(iCanvasFullScreen);


    }

    private void toggleAudioOutput(Player player) {
        AudioOutputControl aocontrol = (AudioOutputControl) player.getControl("com.nokia.mid.media.AudioOutputControl");
        if (aocontrol != null) {
            try {
                System.out.println("Got audio output controller.");
                int nowMode = aocontrol.getOutputMode();
                System.out.println("Got Mode audio output controller." + nowMode);
                if (nowMode != AudioOutputControl.PUBLIC) {
                    System.out.println("Setting audio output mode to PUBLIC");
                    aocontrol.setOutputMode(AudioOutputControl.PUBLIC);
                } else if (nowMode == AudioOutputControl.PUBLIC) {
                    aocontrol.setOutputMode(AudioOutputControl.PRIVATE);
                    System.out.println("Setting audio output mode to Private");
                }
            } catch (Exception ie) {
                ie.printStackTrace();
            }
        } else {
            System.out.println("No audio output controller.");
        }
    }

    private void switchChannel(int aChannelIndex) {
        Service service = (Service) iServices.elementAt(aChannelIndex);
        iController.prepare(service);
        iController.start();
        iCurrentService = iController.getCurrentService();
        updateTitle(" ");

    }
}
