package mbs.views;



import javax.microedition.lcdui.*;

/**
 * Interface for all main views in the application
 */
public interface ViewBase
{
    /**
     * Activates the view. This function is expected to do the basic
     * initialization of the view. After this the view can be displayed
     */
    public void activate();

    /**
     * Deactivates the view, This function is expected to uninitialize
     * any signification data structures that the view currently holds
     * After this the view is not displayed anymore but may be deactivated
     * again.
     */
    public void deactivate();

    /**
     * Returns the displayable of this view
     *
     * @return Displayable of this view
     */
    public Displayable getDisplayable();

    /**
     * Sets the command listener for this view
     *
     * @param aListener The command listener listening commands for this view
     */
    public void setCommandListener(CommandListener aListener);


    /**
     * Handles a specific command provided by the command listener
     *
     * @param aCommand A command to be handled
     * @param aDisplayable Displayable Displaying this view
     */
    public void handleCommand(Command aCommand, Displayable aDisplayable);
}