package mbs.controllers;


import javax.microedition.broadcast.purchase.*;


/**
 * Controls subscriptions.
 */
public class SubscriptionController
{
    // Singleton instance
    private static SubscriptionController sInstance;
    // ID used for getting Purchase Objects.
    private String iId;
    // Selected purchase object.
    private PurchaseObject iSelectedObject;

    /**
     * Returns the singleton instance of this class
     */
    public static SubscriptionController getInstance()
    {
        if (sInstance == null)
        {
            sInstance = new SubscriptionController();
        }

        return sInstance;
    }

    /**
     * Sets ID of program or service to be used when
     * getPurchaseObjects() is called.
     * Setting null will return all purchase objects.
     */
    public void setId(String aId)
    {
        iId = aId;
    }

    /**
     * Returns purchase objects matching the ID
     * set by setId().
     */
    public PurchaseObject[] getPurchaseObjects()
    {
        PurchaseObject[] objects;

        if (iId == null)
        {
            objects = SubscriptionManager.getAllPurchaseObjects();
        }
        else
        {
            objects = SubscriptionManager.getPurchaseObjects(iId);
        }

        return objects;
    }

    /**
     * Returns the default purchase channel.
     */
    public String getDefaultChannel()
    {
        return SubscriptionManager.getDefaultPurchaseChannel();
    }

    /**
     * Returns all supported purchase channels.
     */
    public String[] getSupportedChannels()
    {
        return SubscriptionManager.getSupportedPurchaseChannels();
    }

    /**
     * Selects a purchase object. This object will be used when price options
     * are asked or the object is purchased.
     */
    public void selectPurchaseObject(PurchaseObject aPurchaseObject)
    {
        iSelectedObject = aPurchaseObject;
    }

    /**
     * Returns the selected purchase object or null if no objects are
     * selected.
     */
    public PurchaseObject getSelectedPurchaseObject()
    {
        return iSelectedObject;
    }

    /**
     * Returns available price options for the selected purchase object.
     */
    public PriceOption[] getPriceOptions()
    {
        return iSelectedObject.getPriceOptions();
    }

    /**
     * Purchases the selected purchase object by using the given
     * price option.
     */
    public void purchase(PriceOption aOption)
    {
        iSelectedObject.purchase(aOption);
    }

    /**
     * Requests price options from the selected purchase object.
     */
    public void requestPriceOptions()
    {
        iSelectedObject.requestPriceOptions();
    }

    /**
     * Returns all subscribed purchase objects.
     */
    public PurchaseObject[] getSubscriptions()
    {
        return SubscriptionManager.getSubscriptions();
    }

    /**
     * Un-subscribes the selected purchase object.
     */
    public void unsubscribe()
    {
        iSelectedObject.cancel();
    }

    private SubscriptionController()
    {
    }
}