package mbs.views;




import javax.microedition.lcdui.*;
import javax.microedition.broadcast.ServiceComponent;
import javax.microedition.broadcast.esg.*;
import javax.microedition.broadcast.purchase.*;
import java.io.IOException;
import java.util.Vector;
import mbs.controllers.ServiceGuideController;
import mbs.controllers.SubscriptionController;

/**
 * Shows information about purchase objects.
 */
public class PurchaseObjectsView extends List
        implements ViewBase, ServiceGuideListener, PurchaseListener {
    // Constants

    protected static final String PURCHASED_IMAGE = "/images/Purchased.JPG";
    protected static final String NOTPURCHASED_IMAGE = "/images/Purchase.JPG";
    // Commands
    private Command iCmdBack;
    private Command iCmdShowInfo;
    private Command iCmdListPriceOptions;
    private Command iCmdUnsubscribe;
    protected SubscriptionController iController;
    protected ServiceGuideController iEsgController;
    protected Vector iPurchaseObjects;

    /**
     * Constructor
     */
    PurchaseObjectsView() {
        super("Purchase objects", List.IMPLICIT);

        iPurchaseObjects = new Vector(5);
        iController = SubscriptionController.getInstance();

        iEsgController = ServiceGuideController.getInstance();

        iCmdBack = new Command("Back", Command.BACK, 1);
        iCmdShowInfo = new Command("Show details", Command.SCREEN, 1);
        iCmdListPriceOptions = new Command("List price options", Command.SCREEN, 1);
        iCmdUnsubscribe = new Command("Unsubscribe", Command.SCREEN, 1);

        addCommand(iCmdShowInfo);
        setSelectCommand(iCmdListPriceOptions);
        addCommand(iCmdUnsubscribe);
        addCommand(iCmdBack);
    }

    /**
     * From ServiceGuideListener
     */
    public void serviceGuideUpdated(
            String aEvent,
            ServiceGuideData aServiceGuideData,
            Object aEventData) {
        // Update the list if ESG has updated.
        if (aEvent.equalsIgnoreCase(ServiceGuideListener.SERVICE_GUIDE_BULK_CHANGED)
                || aEvent.equalsIgnoreCase(ServiceGuideListener.PURCHASE_OBJECT_ADDED)
                || aEvent.equalsIgnoreCase(ServiceGuideListener.PURCHASE_OBJECT_DELETED)) {
            reloadPurchaseObjects();
        }
    }

    /**
     * From PurchaseListener.
     */
    public void purchaseResponse(PurchaseObject aSource,
            String aEvent,
            Object aEventData) {
        System.out.println("PurchaseObjectsView.purchaseResponse: " + aEvent);

        ViewBase currentView = ViewHandler.getCurrentView();
        Displayable current = currentView.getDisplayable();


        if (aEvent.startsWith("Cancel")) {
            String message = aEvent;
            if (aEventData instanceof String) {
                message += ", " + aEventData;
                System.out.println("PurchaseObjectsView.purchaseResponse, data="
                        + aEventData);
            }

            AlertNote note = new AlertNote("Cancel response",
                    message,
                    AlertType.INFO,
                    current);
            note.execute();
        }

        if (aEvent.equalsIgnoreCase(PurchaseListener.CANCEL_COMPLETED)
                || aEvent.equalsIgnoreCase(PurchaseListener.PURCHASE_COMPLETED)) {
            reloadPurchaseObjects();
        }
    }

    /**
     * Activates the view
     */
    public void activate() {
        iEsgController.addListener(this);
        reloadPurchaseObjects();
    }

    /**
     * Deactivates the view
     */
    public void deactivate() {
        deleteAll();
    }

    /**
     * Returns the displayable of this view
     *
     * @return Displayable of this view
     */
    public Displayable getDisplayable() {
        return this;
    }

    /**
     * Handles a command which is delegated to this object
     *
     * @param aCommand The Command to be handled
     * @param aDisplayable The displayable which initiated the command
     */
    public void handleCommand(Command aCommand, Displayable aDisplayable) {
        if (aCommand == iCmdBack) {
            ViewHandler.activatePreviousView();
            return;
        }

        int index = getSelectedIndex();
        if (index < 0) {
            // Nothing to do.
            return;
        }

        PurchaseObject object =
                (PurchaseObject) iPurchaseObjects.elementAt(index);

        if (aCommand == iCmdShowInfo) {
            new PurchaseObjectDataInfo(getString(index), this, object);
        } else if (aCommand == iCmdListPriceOptions) {
            object.addPurchaseListener(this);
            iController.selectPurchaseObject(object);
            ViewHandler.activateView(ViewHandler.PRICE_OPTION_VIEW_ID);
        } else if (aCommand == iCmdUnsubscribe) {
            object.addPurchaseListener(this);
            iController.selectPurchaseObject(object);
            iController.unsubscribe();
        }
    }

    /**
     * Asks purchase objects from the engine and updates the list.
     */
    protected synchronized void reloadPurchaseObjects() {
        Image purchasedImage = null;
        Image notPurchasedImage = null;

        Class tempClass = getClass();

        try {
            purchasedImage = Image.createImage(
                    tempClass.getResourceAsStream(PURCHASED_IMAGE));

            notPurchasedImage = Image.createImage(
                    tempClass.getResourceAsStream(NOTPURCHASED_IMAGE));
        } catch (IOException ioe) {
            // Ignored, no icons shown for the user.
            ioe.printStackTrace();
        }

        deleteAll();
        iPurchaseObjects.removeAllElements();

        PurchaseObject[] objects = iController.getPurchaseObjects();

        for (int i = 0; i < objects.length; i++) {
            String name;

            try {
                name = (String) objects[i].getStringValue(
                        CommonMetadataSet.PURCHASE_ITEM_NAME);
            } catch (DataUnavailableException due) {
                name = "(no name)";
            }

            Image image = (objects[i].isPurchased() ? purchasedImage
                    : notPurchasedImage);

            iPurchaseObjects.addElement(objects[i]);
            append(name, image);
        }
    }
}
