package mbs.views;




import javax.microedition.lcdui.*;
import javax.microedition.broadcast.ServiceComponent;
import javax.microedition.broadcast.esg.*;
import javax.microedition.broadcast.purchase.PurchaseObject;
import java.util.Date;

/**
 * Displays information from the specified service guide item
 */
public class ServiceGuideDataInfo extends Form implements CommandListener
{
    // Next displayable
    private Displayable iNextDisplayable;
    // Display of the application
    private Display iDisplay;
    // Service guide data object
    protected ServiceGuideData iData;
    // Commands
    protected Command iCmdBack;

    /**
     * Constructor
     */
    public ServiceGuideDataInfo(
        String aTitle,
        Displayable aNextDisplayable,
        ServiceGuideData aData)
    {
        super(aTitle);

        iNextDisplayable = aNextDisplayable;
        iData = aData;
        iDisplay = ViewHandler.getDisplay();

        iCmdBack = new Command("Back", Command.BACK, 2);

        addCommand(iCmdBack);

        setCommandListener(this);
        iDisplay.setCurrent(this);

        reloadMetadata();
    }

    /**
     * Handles command which is delegated to this object
     * @param aCommand Command to handle
     * @param aDisplayable View owning this view
     */
    public void commandAction(Command aCommand, Displayable aDisplayable)
    {
        if (aCommand == iCmdBack)
        {
            iDisplay.setCurrent(iNextDisplayable);
        }
    }

    /**
     * Clears previous and loads new service guide item's metadata
     * in the UI.
     */
    protected void reloadMetadata()
    {
        deleteAll();
        refreshMetadata();
    }

    /**
     * Refreshes the service guide item's meta data in the UI
     */
    protected void refreshMetadata()
    {
        addServiceMetaData();
        addProgramMetaData();
        addPurchaseMetaData();
        addServiceComponentMetaData();
    }

    private void addServiceMetaData()
    {
        if (!(iData instanceof Service))
        {
            return;
        }

        Attribute[] attributes =
            CommonMetadataSet.getInstance().getValidServiceAttributes();

        for (int i = 0; i < attributes.length; i++)
        {
            addAttributeText(attributes[ i ]);
        }
    }

    private void addProgramMetaData()
    {
        if (!(iData instanceof ProgramEvent))
        {
            return;
        }

        Attribute[] attributes =
            CommonMetadataSet.getInstance().getValidProgramAttributes();

        for (int i = 0; i < attributes.length; i++)
        {
            addAttributeText(attributes[ i ]);
        }
    }

    private void addPurchaseMetaData()
    {
        if (!(iData instanceof PurchaseObject))
        {
            return;
        }

        Attribute[] attributes =
            CommonMetadataSet.getInstance().getValidPurchaseAttributes();

        for (int i = 0; i < attributes.length; i++)
        {
            addAttributeText(attributes[ i ]);
        }
    }

    private void addServiceComponentMetaData()
    {
        if (!(iData instanceof ServiceComponent))
        {
            return;
        }

        Attribute[] attributes =
            CommonMetadataSet.getInstance().getValidServiceComponentAttributes();

        for (int i = 0; i < attributes.length; i++)
        {
            addAttributeText(attributes[ i ]);
        }
    }


    private void addAttributeText(Attribute aAttribute)
    {
        try
        {
            append("\n" + aAttribute.getName() + ":");
            if (aAttribute instanceof StringAttribute)
            {
                append("String:");
                String[] values =
                    iData.getStringValues((StringAttribute)aAttribute);

                for (int k = 0; k < values.length; k++)
                {
                    addText(aAttribute.getName(), values[ k ]);
                }
            }
            else if (aAttribute instanceof DateAttribute)
            {
                append("Date:");
                Date[] values =
                    iData.getDateValues((DateAttribute)aAttribute);

                for (int k = 0; k < values.length; k++)
                {
                    addText(aAttribute.getName(),
                            values[ k ].toString());
                }
            }
            else if (aAttribute instanceof BooleanAttribute)
            {
                append("Bool:");
                boolean[] values =
                    iData.getBooleanValues((BooleanAttribute)aAttribute);

                for (int k = 0; k < values.length; k++)
                {
                    addText(aAttribute.getName(),
                            new Boolean(values[ k ]).toString());
                }
            }
            else if (aAttribute instanceof NumericAttribute)
            {
                append("Numeric:");
                NumericAttribute numAttribute = (NumericAttribute)aAttribute;
                if (numAttribute.getType() == NumericAttribute.FLOATING_POINT_TYPE)
                {
                    append("Float:");
                    double[] values = iData.getDoubleValues(numAttribute);
                    for (int k = 0; k < values.length; k++)
                    {
                        addText(aAttribute.getName(),
                                Double.toString(values[ k ]));
                    }
                }
                else
                {
                    append("Integer:");
                    long[] values = iData.getLongValues(numAttribute);
                    for (int k = 0; k < values.length; k++)
                    {
                        addText(aAttribute.getName(),
                                Long.toString(values[ k ]));
                    }
                }
            }
            else
            {
                append("Unknown Type");
            }
        }
        catch (Exception e)
        {
            System.out.println("ServiceGuideDataInfo.refreshMetadata() - exception: "
                               + e.toString());
            append("Unknown Value\n");
        }
    }

    /**
     * Adds a new TextField to this form
     * @param aTitle Title of the text field
     * @param aValue Value of the text field
     */
    private void addText(String aTitle, String aValue)
    {
        StringItem textField = new StringItem(aTitle + ":", aValue);
        textField.setFont(ViewUtils.getDefaultFont());
        append(textField);
    }
}

// End of File
