package mbs.views;




import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Enumeration;

import javax.microedition.broadcast.connection.BroadcastConnection;
import javax.microedition.broadcast.connection.BroadcastConnectionListener;
import javax.microedition.broadcast.connection.BroadcastFileConnection;
import javax.microedition.io.Connector;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Image;
import javax.microedition.lcdui.List;

public class ShowFilesView extends List implements Runnable,
        BroadcastConnectionListener, CommandListener {

    private Command iBackCommand;
    private Command iSelectCommand;
    private String iFileLocation;
    private BroadcastFileConnection iBfc;

    public ShowFilesView(String aFileLocation) {
        super(aFileLocation, List.EXCLUSIVE);
        iSelectCommand = new Command("Show file as image", Command.ITEM, 1);
        addCommand(iSelectCommand);
        iFileLocation = aFileLocation;
        iBackCommand = new Command("Back", Command.BACK, 1);
        addCommand(iBackCommand);
        setCommandListener(this);
        new Thread(this).start();
    }

    public void run() {
        try {
            iBfc = (BroadcastFileConnection) Connector.open(iFileLocation, Connector.READ);
            iBfc.addListener(this);

            // List does not return anything until the file description table
            // has been received.
            boolean waitingFileList = true;
            while (waitingFileList) {
                Enumeration e = iBfc.list();
                while (e.hasMoreElements()) {
                    BroadcastFileConnection file = (BroadcastFileConnection) e.nextElement();
                    file.addListener(this);
                    addItem(file.getName());
                    waitingFileList = false;
                }
            }
        } catch (Throwable e) {
            addItem(e.toString());
        }
    }

    private void addItem(String aMessage) {
        append(aMessage, null);
    }

    public void update(BroadcastConnection aConnection, String aEvent, Object aData) {
        //BroadcastFileConnection bfc = (BroadcastFileConnection)aConnection;
        System.out.println("BroadcastConnection update ++++ ");
        BroadcastFileConnection con = (BroadcastFileConnection) aConnection;
        try {
            if (!con.isDirectory()) {
                int i = findIndex(con);
                set(i, "A: " + getString(i), getImage(i));
            }
        } catch (IOException e) {
            addItem("IOE: " + e.getMessage());
        }
    }

    private BroadcastFileConnection findConnection(String aName) throws IOException {
        Enumeration e = iBfc.list();
        while (e.hasMoreElements()) {
            BroadcastFileConnection file = (BroadcastFileConnection) e.nextElement();
            if (file.getName().equals(aName)) {
                return file;
            }
        }
        return null;
    }

    private int findIndex(BroadcastFileConnection aConnection) {
        String name = aConnection.getName();
        int count = size();
        for (int i = 0; i < count; i++) {
            if (getString(i).equals(name)) {
                return i;
            }
        }
        return -1;
    }

    private void makeConnection(int aIndex) throws IOException {
        String name = getString(aIndex);
        if (name.startsWith("A: ")) {
            name = name.substring(3);
        }
        BroadcastFileConnection file = findConnection(name);
        InputStream is = file.openInputStream();
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        int value = 0;
        while ((value = is.read()) != -1) {
            baos.write(value);
        }
        baos.flush();
        byte[] imageBytes = baos.toByteArray();
        Image image = Image.createImage(imageBytes, 0, imageBytes.length);
        set(aIndex, getString(aIndex), image);
    }

    public void commandAction(Command aCommand, Displayable aDisplayable) {
        if (aCommand == iBackCommand) {
            ViewHandler.activateView(ViewHandler.SERVICES_VIEW_ID);
        }
        if (aCommand == iSelectCommand) {
            try {
                makeConnection(getSelectedIndex());
            } catch (IOException e) {
                addItem("IO:" + e.getMessage());
            } catch (NullPointerException npe) {
                addItem("Could not find connection");
            }
        }

    }
}
